import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import greenfoot.junitUtils.runner.GreenfootRunner;
import greenfoot.junitUtils.*;

@RunWith(GreenfootRunner.class)
public class BallTest {
    MyWorld world;
    Ball ball;

    @Before
    public void setUp() throws Exception { 
        // GIVEN: The local variables one world and one ball 
        world = new MyWorld();
        ball = world.getObjects(Ball.class).get(0);
    }    


    @Test
    public void testOneAct() throws Exception {
        // WHEN : The action to test is one act cycles
        WorldCreator.runOnce(world);    
    
        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(2, ball.getY());
    }  
 
    @Test
    public void testOneActBis() throws Exception {
        // WHEN : The action to test is one act cycles
        ball.act();    
    
        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(2, ball.getY());
    }  
 
    @Test
    public void testOneActBisBis() throws Exception {
        // WHEN : The action to test is one act cycles
        WorldCreator.runOnce();    
    
        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(2, ball.getY());
    }  
 
    @Test
    public void testTwoAct() throws Exception {
        // WHEN : The action to test is two act cycles
        WorldCreator.runOnce(world);
        WorldCreator.runOnce(world);  

        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(3, ball.getY());
    }   

    @Test
    public void testTwoActBis() throws Exception {
        // WHEN : The action to test is two act cycles
        ball.act();    
        ball.act();     

        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(3, ball.getY());
    }   

    @Test
    public void testTwoActBisBis() throws Exception {
        // WHEN : The action to test is two act cycles
        WorldCreator.runOnce();
        WorldCreator.runOnce();  

        // THEN : The asserts to verify is the ball position
        assertEquals(2, ball.getX());
        assertEquals(3, ball.getY());
    }   


    @Test
    public void testClickedBall() throws Exception {
        // WHEN : The action to test is to click the ball
        EventDispatch.mouseClicked(2, 1);
        WorldCreator.runOnce(world);   
     
        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(1, ball.getY());
    }   
    
    @Test
    public void testClickedBallBis() throws Exception {
        // WHEN : The action to test is to click the ball
        EventDispatch.mouseClicked(2, 1);
        ball.act();     
     
        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(1, ball.getY());
    } 
  
    @Test
    public void testClickedBallBisBis() throws Exception {
        // WHEN : The action to test is to click the ball
        EventDispatch.mouseClicked(2, 1);
        WorldCreator.runOnce();   
     
        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(1, ball.getY());
    }   
    
    @Test
    public void testClickedBallAfterFallOut() throws Exception {        
        // WHEN : The action to test is to click the ball
        WorldCreator.runOnce(world);
        EventDispatch.mouseClicked(2, 2);
        WorldCreator.runOnce(world);

        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(2, ball.getY());
    }   

    @Test
    public void testClickedBallAfterFallOutBis() throws Exception {        
        // WHEN : The action to test is to click the ball
        ball.act();     
        EventDispatch.mouseClicked(2, 2);
        ball.act();     

        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(2, ball.getY());
    }   

    @Test
    public void testClickedBallAfterFallOutBisBis() throws Exception {        
        // WHEN : The action to test is to click the ball
        WorldCreator.runOnce(world);
        EventDispatch.mouseClicked(2, 2);
        WorldCreator.runOnce();

        // THEN : The asserts to verify is the ball position
        assertEquals(3, ball.getX());
        assertEquals(2, ball.getY());
    }   

    @Test
    public void testImageChanged() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(19);
        WorldCreator.runOnce(world);

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png",
                     ball.getImage().getImageFileName());
    } 
  
    @Test
    public void testImageChangedBis() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(19);
        ball.act();     

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png", 
                     ball.getImage().getImageFileName());
    } 
  
    @Test
    public void testImageChangedBisBis() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(19);
        WorldCreator.runOnce();

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png",
                     ball.getImage().getImageFileName());
    } 

    @Test
    public void testImageNotChanged() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(20);
        WorldCreator.runOnce(world);

        // THEN : The asserts to verify is that it is the same image
        assertEquals("gold-ball.png", ball.getImage().getImageFileName());
    }

    @Test
    public void testImageNotChangedBis() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(20);
        ball.act();     

        // THEN : The asserts to verify is that it is the same image
        assertEquals("gold-ball.png", ball.getImage().getImageFileName());
    }   

    @Test
    public void testImageNotChangedBisBis() throws Exception {        
        // WHEN : The action to test is image changed
        Random.set(20);
        WorldCreator.runOnce();

        // THEN : The asserts to verify is that it is the same image
        assertEquals("gold-ball.png", ball.getImage().getImageFileName());
    }

    @Test
    public void testImageChangedAftertwoActCycles() throws Exception {  
        // GIVEN:
        Random.set(20);
        Random.set(20);
        WorldCreator.runOnce(world);

        // WHEN : The action to test is image changed
        Random.set(19);
        ball.act();

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png", ball.getImage().getImageFileName());
    } 

    @Test
    public void testImageChangedAftertwoActCyclesBis() throws Exception {  
        // GIVEN:
        Random.set(20);
        Random.set(20);
        WorldCreator.runOnce();

        // WHEN : The action to test is image changed
        Random.set(19);
        ball.act();

        // THEN : The asserts to verify that the image is changed
        assertEquals("steel-ball.png", ball.getImage().getImageFileName());
    } 

}
